###############################################################################
# makefile для включения в проект.
#
# на входе принимает параметры:
#  TARGET_CPU     - определяет, файлы поддержки какого чипа должны быть использованы
#  CHIP_DIR       - путь к данному файлу относительно директории сборки проекта
#                                (по-умолчанию lib/chip)
# устанавливает следующие параметры:
#  CHIP_SRC                 - исходные файлы на C (без startup)
#  CHIP_DEFS                - определения, которые должны определены при сборке
#                             для включения поддержки чипа
#  CHIP_CFLAGS              - флаги для компиляции C-файлов
#  CHIP_ASFLAGS             - флаги для компиляции файлов ассемблера
#  CHIP_LDFLAGS             - флаги для линковки
#  CHIP_INC_DIRS            - директории с заголовками, которые должны быть добавлены
#                               в список директорий для поиска заголовков проекта
#  CHIP_STARTUP_SRC          - файл startup, если он написан на C
#  CHIP_STARTUP_SRC_ASM      - файл startup, если он написан на ASM
#  CHIP_SPEC_DIR             - директория с файлами для указанного чипа
#
#  Для гетерогенных многоядерных контроллеров параметры для второстепенного
#   ядра сохраняются в переменные с префиксом
#   (например CHIP_DEFS_M0, CHIP_CFLAGS_M0, CHIP_ASFLAGS_M0, CHIP_STARTUP_SRC_M0
#    для ядра cortex-m0 на lpc43xx)
################################################################################



CHIP_DIR ?= ./lib/chip

CHIP_INC_DIRS := $(CHIP_DIR)
CHIP_SPEC_DIR := $(CHIP_DIR)/$(TARGET_CPU)
CHIP_CMSIS_DIR := $(CHIP_DIR)/cmsis

CHIP_SRC:=$(CHIP_SPEC_DIR)/init/chip_init.c


#append port file to sources
ifeq ($(TARGET_CPU), lpc17xx)
    $(error lpc17xx not supported yet)
else ifeq ($(TARGET_CPU), lpc43xx)
    CHIP_INC_DIRS += $(CHIP_CMSIS_DIR)

    CHIP_DEFS     = CORE_M4 CHIP_LPC43XX ARM_MATH_CM4
    CHIP_DEFS_M0  = CORE_M0 CHIP_LPC43XX LPC43XX_CORE_M0APP ARM_MATH_CM0

    CHIP_CFLAGS    = -mcpu=cortex-m4 -mthumb
    CHIP_CFLAGS_M0 = -mcpu=cortex-m0 -mthumb
    CHIP_ASFLAGS = $(CHIP_CFLAGS)
    CHIP_ASFLAGS_M0 = $(CHIP_CFLAGS_M0)
    CHIP_LDFLAGS = $(CHIP_CFLAGS)

    CHIP_STARTUP_SRC     = $(CHIP_SPEC_DIR)/init/startup_gcc.c
    CHIP_STARTUP_SRC_M0  = $(CHIP_SPEC_DIR)/init/startup_m0.c


    CHIP_TOOLCHAIN_TARGET ?= arm-none-eabi-
else ifeq ($(TARGET_CPU), lpc11u6x)
    CHIP_INC_DIRS += $(CHIP_CMSIS_DIR)

    CHIP_DEFS     = CORE_M0PLUS CHIP_LPC11U6X ARM_MATH_CM0PLUS
    CHIP_CFLAGS = -mcpu=cortex-m0 -mthumb
    CHIP_ASFLAGS = $(CHIP_CFLAGS)
    CHIP_LDFLAGS = $(CHIP_CFLAGS)

    CHIP_STARTUP_SRC     = $(CHIP_SPEC_DIR)/init/startup_gcc.c
    CHIP_TOOLCHAIN_TARGET ?= arm-none-eabi-
else ifeq ($(TARGET_CPU), bf609)
    CHIP_DEFS     = CHIP_BF609

    CHIP_STARTUP_SRC_ASM = $(CHIP_SPEC_DIR)/init/chip_startup.s
    CHIP_SRC+=$(CHIP_SPEC_DIR)/chip_interrupt.c

    CHIP_CFLAGS = -mcpu=bf609 -mspecld-anomaly -mcsync-anomaly
    CHIP_ASFLAGS = $(CHIP_CFLAGS)
    CHIP_LDFLAGS = $(CHIP_CFLAGS)
    CHIP_LDRFLAGS = -T BF609



    CHIP_TOOLCHAIN_TARGET ?= bfin-elf-
else
    $(error chip  err: valid chip name must be specified in  TARGET_CPU variable)
endif

CC   = $(CHIP_TOOLCHAIN_TARGET)gcc
CP   = $(CHIP_TOOLCHAIN_TARGET)objcopy
AS   = $(CHIP_TOOLCHAIN_TARGET)gcc -x assembler-with-cpp
SIZE = $(CHIP_TOOLCHAIN_TARGET)size
LDR  = $(CHIP_TOOLCHAIN_TARGET)ldr
HEX  = $(CP) -O ihex
BIN  = $(CP) -O binary
