/*
 * @brief LPC11u6x DMA chip driver
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2013
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licensor disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#ifndef __DMA_11U6X_H_
#define __DMA_11U6X_H_


/** @defgroup DMA_11U6X CHIP: LPC11u6x DMA Controller driver
 * @ingroup CHIP_11U6X_Drivers
 * @{
 */

/**
 * @brief DMA Controller shared registers structure
 */
typedef struct {                    /*!< DMA shared registers structure */
    __IO uint32_t  ENABLESET;        /*!< DMA Channel Enable read and Set for all DMA channels */
    __I  uint32_t  RESERVED0;
    __O  uint32_t  ENABLECLR;        /*!< DMA Channel Enable Clear for all DMA channels */
    __I  uint32_t  RESERVED1;
    __I  uint32_t  ACTIVE;            /*!< DMA Channel Active status for all DMA channels */
    __I  uint32_t  RESERVED2;
    __I  uint32_t  BUSY;            /*!< DMA Channel Busy status for all DMA channels */
    __I  uint32_t  RESERVED3;
    __IO uint32_t  ERRINT;            /*!< DMA Error Interrupt status for all DMA channels */
    __I  uint32_t  RESERVED4;
    __IO uint32_t  INTENSET;        /*!< DMA Interrupt Enable read and Set for all DMA channels */
    __I  uint32_t  RESERVED5;
    __O  uint32_t  INTENCLR;        /*!< DMA Interrupt Enable Clear for all DMA channels */
    __I  uint32_t  RESERVED6;
    __IO uint32_t  INTA;            /*!< DMA Interrupt A status for all DMA channels */
    __I  uint32_t  RESERVED7;
    __IO uint32_t  INTB;            /*!< DMA Interrupt B status for all DMA channels */
    __I  uint32_t  RESERVED8;
    __O  uint32_t  SETVALID;        /*!< DMA Set ValidPending control bits for all DMA channels */
    __I  uint32_t  RESERVED9;
    __O  uint32_t  SETTRIG;            /*!< DMA Set Trigger control bits for all DMA channels */
    __I  uint32_t  RESERVED10;
    __O  uint32_t  ABORT;            /*!< DMA Channel Abort control for all DMA channels */
} LPC_DMA_COMMON_T;

/**
 * @brief DMA Controller shared registers structure
 */
typedef struct {                    /*!< DMA channel register structure */
    __IO uint32_t  CFG;                /*!< DMA Configuration register */
    __I  uint32_t  CTLSTAT;            /*!< DMA Control and status register */
    __IO uint32_t  XFERCFG;            /*!< DMA Transfer configuration register */
    __I  uint32_t  RESERVED;
} LPC_DMA_CHANNEL_T;

/* DMA channel mapping - each channel is mapped to an individual peripheral
   and direction or a DMA imput mux trigger */
typedef enum {
    DMAREQ_SSP0_RX = 0,                /*!< SSP0 receive DMA channel */
    DMA_CH0 = DMAREQ_SSP0_RX,
    DMAREQ_SSP0_TX,                    /*!< SSP0 transmit DMA channel */
    DMA_CH1 = DMAREQ_SSP0_TX,
    DMAREQ_SSP1_RX,                    /*!< SSP1 receive DMA channel */
    DMA_CH2 = DMAREQ_SSP1_RX,
    DMAREQ_SSP1_TX,                    /*!< SSP1 transmit DMA channel */
    DMA_CH3 = DMAREQ_SSP1_TX,
    DMAREQ_USART0_RX,                    /*!< USART0 receive DMA channel */
    DMA_CH4 = DMAREQ_USART0_RX,
    DMAREQ_USART0_TX,                    /*!< USART0 transmit DMA channel */
    DMA_CH5 = DMAREQ_USART0_TX,
    DMAREQ_USART1_RX,                    /*!< USART1 transmit DMA channel */
    DMA_CH6 = DMAREQ_USART1_RX,
    DMAREQ_USART1_TX,                    /*!< USART1 transmit DMA channel */
    DMA_CH7 = DMAREQ_USART1_TX,
    DMAREQ_USART2_RX,                    /*!< USART2 transmit DMA channel */
    DMA_CH8 = DMAREQ_USART2_RX,
    DMAREQ_USART2_TX,                    /*!< USART2 transmit DMA channel */
    DMA_CH9 = DMAREQ_USART2_TX,
    DMAREQ_USART3_RX,                    /*!< USART3 transmit DMA channel */
    DMA_CH10 = DMAREQ_USART3_RX,
    DMAREQ_USART3_TX,                    /*!< USART3 transmit DMA channel */
    DMA_CH11 = DMAREQ_USART3_TX,
    DMAREQ_USART4_RX,                    /*!< USART4 transmit DMA channel */
    DMA_CH12 = DMAREQ_USART4_RX,
    DMAREQ_USART4_TX,                    /*!< USART4 transmit DMA channel */
    DMA_CH13 = DMAREQ_USART4_TX,
    DMAREQ_RESERVED_14,
    DMA_CH14 = DMAREQ_RESERVED_14,
    DMAREQ_RESERVED_15,
    DMA_CH15 = DMAREQ_RESERVED_15
} DMA_CHID_T;

/* On LPC412x, Max DMA channel is 22 */
#define MAX_DMA_CHANNEL            (DMA_CH15 + 1)

/**
 * @brief DMA Controller register block structure
 */
typedef struct {                    /*!< DMA Structure */
    __IO uint32_t  CTRL;            /*!< DMA control register */
    __I  uint32_t  INTSTAT;            /*!< DMA Interrupt status register */
    __IO uint32_t  SRAMBASE;        /*!< DMA SRAM address of the channel configuration table */
    __I  uint32_t  RESERVED2[5];
    LPC_DMA_COMMON_T DMACOMMON[1];    /*!< DMA shared channel (common) registers */
    __I  uint32_t  RESERVED0[225];
    LPC_DMA_CHANNEL_T DMACH[MAX_DMA_CHANNEL];    /*!< DMA channel registers */
} LPC_DMA_T;

/** @defgroup DMA_COMMONDRV_11U6X CHIP: LPC11u6x DMA Controller driver common functions
 * @{
 */

/* DMA channel source/address/next descriptor */
typedef struct {
    uint32_t  xfercfg;        /*!< Transfer configuration (only used in linked lists and ping-pong configs) */
    uint32_t  source;        /*!< DMA transfer source end address */
    uint32_t  dest;            /*!< DMA transfer desintation end address */
    uint32_t  next;            /*!< Link to next DMA descriptor, must be 16 byte aligned */
} DMA_CHDESC_T;

/**
 * @brief DMA trigger pin muxing structure
 */
typedef struct {                    /*!< DMA trigger pin muxing register structure */
    __IO uint32_t  DMA_ITRIG_INMUX[MAX_DMA_CHANNEL];    /*!< Trigger input select register for DMA channels */
} LPC_DMATRIGMUX_T;

/* DMA triggers that can mapped to DMA channels */
typedef enum {
    DMATRIG_ADC0_SEQA_IRQ = 0,            /*!< ADC0 sequencer A interrupt as trigger */
    DMATRIG_ADC0_SEQB_IRQ,                /*!< ADC0 sequencer B interrupt as trigger */
    DMATRIG_CT16B0_MAT0,                /*!< 16-bit counter/timer 0 interrupt as trigger */
    DMATRIG_CT16B1_MAT0,                /*!< 16-bit counter/timer 1 interrupt as trigger */
    DMATRIG_CT32B0_MAT0,                /*!< 32-bit counter/timer 0 interrupt as trigger */
    DMATRIG_CT32B1_MAT0,                /*!< 32-bit counter/timer 1 interrupt as trigger */
    DMATRIG_PINT0,                        /*!< Pin interrupt 0 as trigger */
    DMATRIG_PINT1,                        /*!< Pin interrupt 1 as trigger */
    DMATRIG_SCT0_DMA0,                    /*!< SCT 0, DMA 0 as trigger */
    DMATRIG_SCT0_DMA1,                    /*!< SCT 1, DMA 1 as trigger */
    DMATRIG_SCT1_DMA0,                    /*!< SCT 0, DMA 0 as trigger */
    DMATRIG_SCT1_DMA1                    /*!< SCT 1, DMA 1 as trigger */
} DMA_TRIGSRC_T;

/* Support definitions for setting the configuration of a DMA channel. You
   will need to get more information on these options from the User manual. */
#define DMA_CFG_PERIPHREQEN     (1 << 0)    /*!< Enables Peripheral DMA requests */
#define DMA_CFG_HWTRIGEN        (1 << 1)    /*!< Use hardware triggering via imput mux */
#define DMA_CFG_TRIGPOL_LOW     (0 << 4)    /*!< Hardware trigger is active low or falling edge */
#define DMA_CFG_TRIGPOL_HIGH    (1 << 4)    /*!< Hardware trigger is active high or rising edge */
#define DMA_CFG_TRIGTYPE_EDGE   (0 << 5)    /*!< Hardware trigger is edge triggered */
#define DMA_CFG_TRIGTYPE_LEVEL  (1 << 5)    /*!< Hardware trigger is level triggered */
#define DMA_CFG_TRIGBURST_SNGL  (0 << 6)    /*!< Single transfer. Hardware trigger causes a single transfer */
#define DMA_CFG_TRIGBURST_BURST (1 << 6)    /*!< Burst transfer (see UM) */
#define DMA_CFG_BURSTPOWER_1    (0 << 8)    /*!< Set DMA burst size to 1 transfer */
#define DMA_CFG_BURSTPOWER_2    (1 << 8)    /*!< Set DMA burst size to 2 transfers */
#define DMA_CFG_BURSTPOWER_4    (2 << 8)    /*!< Set DMA burst size to 4 transfers */
#define DMA_CFG_BURSTPOWER_8    (3 << 8)    /*!< Set DMA burst size to 8 transfers */
#define DMA_CFG_BURSTPOWER_16   (4 << 8)    /*!< Set DMA burst size to 16 transfers */
#define DMA_CFG_BURSTPOWER_32   (5 << 8)    /*!< Set DMA burst size to 32 transfers */
#define DMA_CFG_BURSTPOWER_64   (6 << 8)    /*!< Set DMA burst size to 64 transfers */
#define DMA_CFG_BURSTPOWER_128  (7 << 8)    /*!< Set DMA burst size to 128 transfers */
#define DMA_CFG_BURSTPOWER_256  (8 << 8)    /*!< Set DMA burst size to 256 transfers */
#define DMA_CFG_BURSTPOWER_512  (9 << 8)    /*!< Set DMA burst size to 512 transfers */
#define DMA_CFG_BURSTPOWER_1024 (10 << 8)    /*!< Set DMA burst size to 1024 transfers */
#define DMA_CFG_BURSTPOWER(n)   ((n) << 8)    /*!< Set DMA burst size to 2^n transfers, max n=10 */
#define DMA_CFG_SRCBURSTWRAP    (1 << 14)    /*!< Source burst wrapping is enabled for this DMA channel */
#define DMA_CFG_DSTBURSTWRAP    (1 << 15)    /*!< Destination burst wrapping is enabled for this DMA channel */
#define DMA_CFG_CHPRIORITY(p)   ((p) << 16)    /*!< Sets DMA channel priority, min 0 (highest), max 3 (lowest) */

/* DMA channel control and status register definitions */
#define DMA_CTLSTAT_VALIDPENDING    (1 << 0)    /*!< Valid pending flag for this channel */
#define DMA_CTLSTAT_TRIG            (1 << 2)    /*!< Trigger flag. Indicates that the trigger for this channel is currently set */

/* DMA channel transfer configuration registers definitions */
#define DMA_XFERCFG_CFGVALID        (1 << 0)    /*!< Configuration Valid flag */
#define DMA_XFERCFG_RELOAD          (1 << 1)    /*!< Indicates whether the channels control structure will be reloaded when the current descriptor is exhausted */
#define DMA_XFERCFG_SWTRIG          (1 << 2)    /*!< Software Trigger */
#define DMA_XFERCFG_CLRTRIG         (1 << 3)    /*!< Clear Trigger */
#define DMA_XFERCFG_SETINTA         (1 << 4)    /*!< Set Interrupt flag A for this channel to fire when descriptor is complete */
#define DMA_XFERCFG_SETINTB         (1 << 5)    /*!< Set Interrupt flag B for this channel to fire when descriptor is complete */
#define DMA_XFERCFG_WIDTH_8         (0 << 8)    /*!< 8-bit transfers are performed */
#define DMA_XFERCFG_WIDTH_16        (1 << 8)    /*!< 16-bit transfers are performed */
#define DMA_XFERCFG_WIDTH_32        (2 << 8)    /*!< 32-bit transfers are performed */
#define DMA_XFERCFG_SRCINC_0        (0 << 12)    /*!< DMA source address is not incremented after a transfer */
#define DMA_XFERCFG_SRCINC_1        (1 << 12)    /*!< DMA source address is incremented by 1 (width) after a transfer */
#define DMA_XFERCFG_SRCINC_2        (2 << 12)    /*!< DMA source address is incremented by 2 (width) after a transfer */
#define DMA_XFERCFG_SRCINC_4        (3 << 12)    /*!< DMA source address is incremented by 4 (width) after a transfer */
#define DMA_XFERCFG_DSTINC_0        (0 << 14)    /*!< DMA destination address is not incremented after a transfer */
#define DMA_XFERCFG_DSTINC_1        (1 << 14)    /*!< DMA destination address is incremented by 1 (width) after a transfer */
#define DMA_XFERCFG_DSTINC_2        (2 << 14)    /*!< DMA destination address is incremented by 2 (width) after a transfer */
#define DMA_XFERCFG_DSTINC_4        (3 << 14)    /*!< DMA destination address is incremented by 4 (width) after a transfer */
#define DMA_XFERCFG_XFERCOUNT(n)    ((n - 1) << 16)    /*!< DMA transfer count in 'transfers', between (0)1 and (1023)1024 */



#endif /* __DMA_11U6X_H_ */
