/*
 * @brief LPC18xx/43xx State Configurable Timer (SCT) driver
 *
 * @note
 * Copyright(C) NXP Semiconductors, 2012
 * All rights reserved.
 *
 * @par
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * LPC products.  This software is supplied "AS IS" without any warranties of
 * any kind, and NXP Semiconductors and its licenser disclaim any and
 * all warranties, express or implied, including all implied warranties of
 * merchantability, fitness for a particular purpose and non-infringement of
 * intellectual property rights.  NXP Semiconductors assumes no responsibility
 * or liability for the use of the software, conveys no license or rights under any
 * patent, copyright, mask work right, or any other intellectual property rights in
 * or to any products. NXP Semiconductors reserves the right to make changes
 * in the software without notification. NXP Semiconductors also makes no
 * representation or warranty that such application will be suitable for the
 * specified use without further testing or modification.
 *
 * @par
 * Permission to use, copy, modify, and distribute this software and its
 * documentation is hereby granted, under NXP Semiconductors' and its
 * licensor's relevant copyrights in the software, without fee, provided that it
 * is used in conjunction with NXP Semiconductors microcontrollers.  This
 * copyright, permission, and disclaimer notice must appear in all copies of
 * this code.
 */

#ifndef __SCT_18XX_43XX_H_
#define __SCT_18XX_43XX_H_

/** @defgroup SCT_18XX_43XX CHIP: LPC18xx/43xx State Configurable Timer driver
 * @ingroup CHIP_18XX_43XX_Drivers
 * @{
 */

/*
 * @brief SCT Module configuration
 */
#define CONFIG_SCT_nEV   (16)            /*!< Number of events */
#define CONFIG_SCT_nRG   (16)            /*!< Number of match/compare registers */
#define CONFIG_SCT_nOU   (16)            /*!< Number of outputs */

/**
 * @brief State Configurable Timer register block structure
 */
typedef struct {
    __IO  uint32_t CONFIG;              /*!< Configuration Register */
    union {
        __IO uint32_t CTRL_U;           /*!< Control Register */
        struct {
            __IO uint16_t CTRL_L;       /*!< Low control register */
            __IO uint16_t CTRL_H;       /*!< High control register */
        };

    };

    __IO uint16_t LIMIT_L;              /*!< limit register for counter L */
    __IO uint16_t LIMIT_H;              /*!< limit register for counter H */
    __IO uint16_t HALT_L;               /*!< halt register for counter L */
    __IO uint16_t HALT_H;               /*!< halt register for counter H */
    __IO uint16_t STOP_L;               /*!< stop register for counter L */
    __IO uint16_t STOP_H;               /*!< stop register for counter H */
    __IO uint16_t START_L;              /*!< start register for counter L */
    __IO uint16_t START_H;              /*!< start register for counter H */
    uint32_t RESERVED1[10];             /*!< 0x03C reserved */
    union {
        __IO uint32_t COUNT_U;          /*!< counter register */
        struct {
            __IO uint16_t COUNT_L;      /*!< counter register for counter L */
            __IO uint16_t COUNT_H;      /*!< counter register for counter H */
        };

    };

    __IO uint16_t STATE_L;              /*!< state register for counter L */
    __IO uint16_t STATE_H;              /*!< state register for counter H */
    __I  uint32_t INPUT;                /*!< input register */
    __IO uint16_t REGMODE_L;            /*!< match - capture registers mode register L */
    __IO uint16_t REGMODE_H;            /*!< match - capture registers mode register H */
    __IO uint32_t OUTPUT;               /*!< output register */
    __IO uint32_t OUTPUTDIRCTRL;        /*!< output counter direction Control Register */
    __IO uint32_t RES;                  /*!< conflict resolution register */
    __IO uint32_t DMA0REQUEST;          /*!< DMA0 Request Register */
    __IO uint32_t DMA1REQUEST;          /*!< DMA1 Request Register */
    uint32_t RESERVED2[35];
    __IO uint32_t EVEN;                 /*!< event enable register */
    __IO uint32_t EVFLAG;               /*!< event flag register */
    __IO uint32_t CONEN;                /*!< conflict enable register */
    __IO uint32_t CONFLAG;              /*!< conflict flag register */
    union {
        __IO union {                    /*!< ... Match / Capture value */
            uint32_t U;                 /*!<       SCTMATCH[i].U  Unified 32-bit register */
            struct {
                uint16_t L;             /*!<       SCTMATCH[i].L  Access to L value */
                uint16_t H;             /*!<       SCTMATCH[i].H  Access to H value */
            };

        } MATCH[CONFIG_SCT_nRG];

        __I union {
            uint32_t U;                 /*!<       SCTCAP[i].U  Unified 32-bit register */
            struct {
                uint16_t L;             /*!<       SCTCAP[i].L  Access to L value */
                uint16_t H;             /*!<       SCTCAP[i].H  Access to H value */
            };

        } CAP[CONFIG_SCT_nRG];

    };

    uint32_t RESERVED3[32 - CONFIG_SCT_nRG];        /*!< ...-0x17C reserved */
    union {
        __IO uint16_t MATCH_L[CONFIG_SCT_nRG];        /*!< 0x180-... Match Value L counter */
        __I  uint16_t CAP_L[CONFIG_SCT_nRG];        /*!< 0x180-... Capture Value L counter */
    };

    uint16_t RESERVED4[32 - CONFIG_SCT_nRG];        /*!< ...-0x1BE reserved */
    union {
        __IO uint16_t MATCH_H[CONFIG_SCT_nRG];        /*!< 0x1C0-... Match Value H counter */
        __I  uint16_t CAP_H[CONFIG_SCT_nRG];        /*!< 0x1C0-... Capture Value H counter */
    };

    uint16_t RESERVED5[32 - CONFIG_SCT_nRG];        /*!< ...-0x1FE reserved */
    union {
        __IO union {                    /*!< 0x200-... Match Reload / Capture Control value */
            uint32_t U;                    /*!<       SCTMATCHREL[i].U  Unified 32-bit register */
            struct {
                uint16_t L;                /*!<       SCTMATCHREL[i].L  Access to L value */
                uint16_t H;                /*!<       SCTMATCHREL[i].H  Access to H value */
            };

        } MATCHREL[CONFIG_SCT_nRG];

        __IO union {
            uint32_t U;                    /*!<       SCTCAPCTRL[i].U  Unified 32-bit register */
            struct {
                uint16_t L;                /*!<       SCTCAPCTRL[i].L  Access to L value */
                uint16_t H;                /*!<       SCTCAPCTRL[i].H  Access to H value */
            };

        } CAPCTRL[CONFIG_SCT_nRG];

    };

    uint32_t RESERVED6[32 - CONFIG_SCT_nRG];        /*!< ...-0x27C reserved */
    union {
        __IO uint16_t MATCHREL_L[CONFIG_SCT_nRG];    /*!< 0x280-... Match Reload value L counter */
        __IO uint16_t CAPCTRL_L[CONFIG_SCT_nRG];    /*!< 0x280-... Capture Control value L counter */
    };

    uint16_t RESERVED7[32 - CONFIG_SCT_nRG];        /*!< ...-0x2BE reserved */
    union {
        __IO uint16_t MATCHREL_H[CONFIG_SCT_nRG];    /*!< 0x2C0-... Match Reload value H counter */
        __IO uint16_t CAPCTRL_H[CONFIG_SCT_nRG];    /*!< 0x2C0-... Capture Control value H counter */
    };

    uint16_t RESERVED8[32 - CONFIG_SCT_nRG];        /*!< ...-0x2FE reserved */
    __IO struct {                        /*!< 0x300-0x3FC  SCTEVENT[i].STATE / SCTEVENT[i].CTRL*/
        uint32_t STATE;                    /*!< Event State Register */
        uint32_t CTRL;                    /*!< Event Control Register */
    } EVENT[CONFIG_SCT_nEV];

    uint32_t RESERVED9[128 - 2 * CONFIG_SCT_nEV];    /*!< ...-0x4FC reserved */
    __IO struct {                        /*!< 0x500-0x57C  SCTOUT[i].SET / SCTOUT[i].CLR */
        uint32_t SET;                    /*!< Output n Set Register */
        uint32_t CLR;                    /*!< Output n Clear Register */
    } OUT[CONFIG_SCT_nOU];

    uint32_t RESERVED10[191 - 2 * CONFIG_SCT_nOU];    /*!< ...-0x7F8 reserved */
    __I  uint32_t MODULECONTENT;        /*!< 0x7FC Module Content */
} LPC_SCT_T;

/*
 * @brief Macro defines for SCT configuration register
 */
#define SCT_CONFIG_16BIT_COUNTER        0x00000000      /*!< Operate as 2 16-bit counters */
#define SCT_CONFIG_32BIT_COUNTER        0x00000001      /*!< Operate as 1 32-bit counter */

#define SCT_CONFIG_CLKMODE_BUSCLK       (0x0 << 1)      /*!< Bus clock */
#define SCT_CONFIG_CLKMODE_SCTCLK       (0x1 << 1)      /*!< SCT clock */
#define SCT_CONFIG_CLKMODE_INCLK        (0x2 << 1)      /*!< Input clock selected in CLKSEL field */
#define SCT_CONFIG_CLKMODE_INEDGECLK    (0x3 << 1)      /*!< Input clock edge selected in CLKSEL field */

#define SCT_CONFIG_NORELOADL_U          (0x1 << 7)      /*!< Operate as 1 32-bit counter */
#define SCT_CONFIG_NORELOADH            (0x1 << 8)      /*!< Operate as 1 32-bit counter */
#define SCT_CONFIG_AUTOLIMIT_L          (0x1 << 17)     /*!< Limits counter(L) based on MATCH0 */
#define SCT_CONFIG_AUTOLIMIT_H          (0x1 << 18)     /*!< Limits counter(L) based on MATCH0 */

/*
 * @brief Macro defines for SCT control register
 */
#define SCT_CTRL_STOP_L                 (1 << 1)                /*!< Stop low counter */
#define SCT_CTRL_HALT_L                 (1 << 2)                /*!< Halt low counter */
#define SCT_CTRL_CLRCTR_L               (1 << 3)                /*!< Clear low or unified counter */
#define SCT_CTRL_BIDIR_L(x)             (((x) & 0x01) << 4)     /*!< Bidirectional bit */
#define SCT_CTRL_PRE_L(x)               (((x) & 0xFF) << 5)     /*!< Prescale clock for low or unified counter */

#define SCT_CTRL_STOP_H                 (1 << 17)               /*!< Stop high counter */
#define SCT_CTRL_HALT_H                 (1 << 18)               /*!< Halt high counter */
#define SCT_CTRL_CLRCTR_H               (1 << 19)               /*!< Clear high counter */
#define SCT_CTRL_BIDIR_H(x)             (((x) & 0x01) << 20)
#define SCT_CTRL_PRE_H(x)               (((x) & 0xFF) << 21)    /*!< Prescale clock for high counter */

/*
 * @brief Macro defines for SCT Conflict resolution register
 */
#define SCT_RES_NOCHANGE                (0)
#define SCT_RES_SET_OUTPUT              (1)
#define SCT_RES_CLEAR_OUTPUT            (2)
#define SCT_RES_TOGGLE_OUTPUT           (3)

/*
 * @brief Macro defines for SCT event control registers
 */
#define SCT_EVCTRL_MATCHSEL(x)          (((x) & 0x0F) << 0)     /*!< Match number used by this event */
#define SCT_EVCTRL_HEVENT               (1U << 4)               /*!< 1: use high counter state & match */
#define SCT_EVCTRL_OUTSEL               (1U << 5)               /*!< 0: IOSEL selects input, 1: IOSEL selects output */
#define SCT_EVCTRL_IOSEL(x)             (((x) & 0x0F) << 6)     /*!< input/ouput used by this event */
#define SCT_EVCTRL_IOCOND_LOW           (0U << 10)              /*!<  */
#define SCT_EVCTRL_IOCOND_RISE          (1U << 10)              /*!<  */
#define SCT_EVCTRL_IOCOND_FALL          (2U << 10)              /*!<  */
#define SCT_EVCTRL_IOCOND_HIGH          (3U << 10)              /*!<  */
#define SCT_EVCTRL_COMBMODE_OR          (0U << 12)              /*!< match OR i/o must occur */
#define SCT_EVCTRL_COMBMODE_MATCH       (1U << 12)              /*!< uses match only */
#define SCT_EVCTRL_COMBMODE_IO          (2U << 12)              /*!< uses i/o only */
#define SCT_EVCTRL_COMBMODE_AND         (3U << 12)              /*!< match AND i/o must occur */
#define SCT_EVCTRL_STATELD              (1U << 14)              /*!< 0: STATEV is added to state, 1: overwrites it */
#define SCT_EVCTRL_STATEV(x)            (((x) & 0x1F) << 15)    /*!< state value to add/set */
#define SCT_EVCTRL_STATE_ADD(x)         SCT_EVCTRL_STATEV(x)
#define SCT_EVCTRL_STATE_SET(x)         SCT_EVCTRL_STATEV(x) | SCT_EVCTRL_STATELD
#define SCT_EVCTRL_MATCHMEM             (1U << 20)              /*!< 1: keep match event active to end of counting */
#define SCT_EVCTRL_DIRECTION_ANY        (0U << 21)              /*!< event occurs in any direction */
#define SCT_EVCTRL_DIRECTION_UP         (1U << 21)              /*!< event occurs only when counting up */
#define SCT_EVCTRL_DIRECTION_DOWN       (2U << 21)              /*!< event occurs only when counting down */

/**
 * @}
 */

#endif /* __SCT_18XX_43XX_H_ */
