 /**************************************************************************//**
  @defgroup lip_conf Конфигурация стека
  @brief Настройки стека для адаптации его под требования конкретного проекта

  Определенный проект для определенной встаиваемой системы имеет свои требования
    к используемым ресурсам стеком TCP/IP и к реализуемым стеком функциям.
    В связи с этим, часть параметров стека определяется в отдельном файле
    конфигурации (lip_conf.h) в виде конфигурационных макросов.

    Данный файл должен быть скопирован в каждый проект и изменен в соответствии
    с требованиями данного проекта.

    В файле конфигурации содержится набор макросов типа @c LIP_USE_MODULENAME,
    определяющий какие модули будут включены в стек при компиляции. Далее
    внутри директив \#ifdef LIP_USE_MODULENAME   \#endif идут переметры для
    настройки конкретного модуля в виде макросов LIP_MODULENAME_PARAMNAME.
    Некоторые модули имеют зависимости от других, поэтому при их включении
    необходимо, чтобы были включены и модули из списка зависимостей (смотри
    описание модулей), иначе произойдет ошибка при компиляции с соответствующим
    сообщением.

    Часть параметров, относящаяся либо ко всему стеку в целом или к модулям,
    которые всегда включены (такие как @ref lip_ip или @ref lip_arp), идут
    без условных директив в начале файла конфигурации.

    Настройка файла позволяет оптимизировать размер памяти (как кода, так и данных),
    используемый стеком, для конкретного проекта, а так же установить нужные
    временные параметры или другие параметры работы определенных протоколов.
  *****************************************************************************/

/***************************************************************************//**
  @addtogroup lip_conf
  @{
  @file lip_conf.h
  @brief Данный файл содержит пример настроек TCP/IP стека lip.
    Он должен быть скопирован в каждый проект и изменен в соответствии
    с требованиями конкретного проекта.
  @author Borisov Alexey <borisov@lcard.ru>
 ******************************************************************************/

#ifndef LIP_CONF_H_
#define LIP_CONF_H_

#include "lip_log.h"

extern int lprintf(char* fmt, ...);
/** Данный макрос, если определн, определяет функцию для вывода отладочных
    сообщений стека. Первым параметром передается константа с типом модуля,
    который сгенерировал сообщения, второй --- уровень сообщения (ошибка, предупреждение,
    отладочное сообщение и т.д.), третий --- код ошибки или 0. Остальные параметры
    соответствуют параметрам функции printf() (формат и произвольный набор
    параметров, зависящий от формата). Если не определен, то код для печати
    отладочных сообщений будет выкинут из стека */
#define lip_printf(module, level, err, ...) do { if ((module != LIP_LOG_MODULE_TCP) || \
        (level < LIP_LOG_LVL_DEBUG_HIGH)) \
        { lprintf(__VA_ARGS__);} } while(0)


/** Время между последовательными проверками состояния линка (мс) --- подключен ли кабель */
#define LIP_LINK_CHECK_TOUT  500

/** Если определено, то в случае потери линка, стек пытается заново установить
  связь сам и при появлении связи запускает автоматически все модули.
  Если не определно --- то при потере связи стек переходит в состояние ошибки
  и приложение должено закрыть стек и заново его проинициализировать для повторного
  использования после восстановления связи */
#define LIP_LINK_AUTO_RECONNECT

/** Определение данного макроса указывает, что архитектура поддерживает доступ
    к невыровненным словам. Это позволяет некторые записи лучше упаковать
    (в частности отчеты в @ref lip_igmp) */
#define LIP_SUPPORT_UNALIGN_ACCESS

/** Определение этого макроса включает модуль @ref lip_rand для генерации случайных
    чисел */
#define LIP_USE_RAND_GEN
#ifdef LIP_USE_RAND_GEN
    /** если определено, то lip_rand_range() может генерировать числа
        в диапазоне от [0, 2^32). Иначе --- только [0, 2^16)  */
    #define LIP_RAND_UINT32_RANGE_SUPPORT
#endif




/* -------------------------- Настройки PHY-уровня ---------------------------*/
/** Поддержка PHY TI DP83848J */
#define LIP_PHY_SUPPORT_DP83848J
/** Поддержка PHY SMSC LAN8720*/
#define LIP_PHY_SUPPORT_LAN8720

/** Таймаут в мс на автоопределение параметров канала связи */
#define LIP_PHY_AUTONEG_TIMEOUT  5000

/** Если определено, перед инициализацией выполняется аппаратный сброс PHY */
#define LIP_PHY_HARDWARE_RESET
#ifdef LIP_PHY_HARDWARE_RESET
    /** Время аппаратного сброса PHY в мс */
    #define LIP_PHY_RESET_TIME       100

    /** Настройка пинов для сброса Phy */
    #define LIP_PHY_RESET_CFG() 
    /** Макрос переводит PHY в состояние сброса */
    #define LIP_PHY_RESET_SET() 
    /** Макрос для возвращение PHY из состояние сброса в рабочее */
    #define LIP_PHY_RESET_CLR()
#endif


/*-------------------------------- настройки IP ------------------------------*/
/** Время жизни (TTL) по-умолчанию для @ref lip_ipv4 пакетов. Используется, если
    время жизни на задано явно верхним уровнем */
#define LIP_IP_TTL 150

/** Данный макрос включает логику сборки входных фрагментированных пакетов.
    Если он не определен, то все фрагментированные пакеты отбрасываются.
    Включение этого модуля требует дополнительной памяти порядка
    #LIP_IPV4_REASSEMBLY_BUF_CNT*(#LIP_IPV4_REASSEMBLY_MAX_DATA_SIZE+70) байт*/
#define LIP_USE_IPV4_REASSEMBLY

#ifdef LIP_USE_IPV4_REASSEMBLY
    /** Определяет количество используемых буферов для сборки дейтаграммы, т.е.
        максимальное количество незавершенных сборок IP-дейтаграмм. При приходе
        фрагмента новой дейтаграммы, для сохранения которого нет свободного буфера,
        фрагмент будет отброшен */
    #define LIP_IPV4_REASSEMBLY_BUF_CNT 4

    /** Максимальный размер собираемой дейтаграммы. Если всего полезных данных
        в собранной фрагментированной IP-дейтаграмме больше указанного размера,
        то дейтаграмма будет отброшена */
    #define LIP_IPV4_REASSEMBLY_MAX_DATA_SIZE 1500
#endif

/*-------------------------------- настройки ARP -----------------------------*/
/** Количество записей в таблице ARP, хранящей соответствие IPv4 адресов
    и MAC-адресов */
#define LIP_ARP_IPV4_TABLE_LENGTH  4
/** Время (в мс) соответствующее увеличению "возраста" записи из ARP-таблицы
    на единицу */
#define LIP_ARP_AGE_INC_TIME       500
/** Макрос определяет "возраст" ARP-записи, при достижении которого она
    выкидывается из таблицы (умноженное на #LIP_ARP_AGE_INC_TIME определяет
    время жизни записи в таблице) */
#define LIP_ARP_DISCARD_AGE        120

/** Если макрос определен, то в модуле @ref lip_ipv4  создается буфера указанного
    размера, который используется, чтобы сохранить пакет, ожидающий разрешения
    через ARP адреса назначения. Если макрос на определен, то отправляемый пакет,
    для которого нет соответствия IP-адреса назначения и MAC-адреса всегда
    выбрасывается */
#define LIP_IP_WAIT_ARP_BUF_SIZE   1500

#ifdef LIP_IP_WAIT_ARP_BUF_SIZE
    /** Макрос определяет время (в мс), которое пакет, отложенный до разрешения
        адреса назначения, сохраняется в буфере. Если за указанное время адрес не
        будет разрешен, то пакет выбрасывается и буфер освобождается */
    #define  LIP_IP_WAIT_ARP_TIME  2000
#endif


/***************************************************************************//**
    @defgroup lip_conf_udp Конфигурация модуля UDP
    @brief Набор параметров, задающих конфигурацию модуля @ref lip_udp
    @addtogroup lip_conf_udp
    @{
********************************************************************************/

/** При определении данного макроса будет включена сборка модуля @ref lip_udp */
#define LIP_USE_UDP
#ifdef LIP_USE_UDP
    /** максимальное кол-во портов, которое можно прослушивать одновременно */
    #define LIP_UDP_MAX_DYN_PORTS    3
#endif
/** @} */




/***************************************************************************//**
    @defgroup lip_conf_tcp Конфигурация модуля TCP
    @brief Набор параметров, задающих конфигурацию модуля @ref lip_tcp
    @addtogroup lip_conf_tcp
    @{
********************************************************************************/

/** При определении данного макроса будет включена сборка модуля @ref lip_tcp */
#define LIP_USE_TCP
#ifdef LIP_USE_TCP
    /** Максимальное количество пользовательских сокетов TCP --- сокетов,
        с которыми идет работа напрямую через функции TCP, без учета сокетов,
        используемых модулями самого стека */
    #define LIP_TCP_USER_SOCKET_CNT      1
    /** Максимальный размер сегмента TCP на прием */
    #define LIP_TCP_RCV_MSS           1460
    /** Максимальный размер сегмента TCP на передачу для данной сети
        (выбирается минимальный между объявленным другой стороной и этим значением) */
    #define LIP_TCP_MAX_SEND_SIZE     1460

    /** Количество повторных передач сегмента по таймеру, после которого соединение
        будет разорвано */
    #define LIP_TCP_RETRANS_MAX_CNT   5

    /** Максимальное количество сегментов в очереди для повторной передачи
        Определяет количество сегментов, которое может быть послано другой стороне,
        но не подтверждено (Flight segments) */
    #define LIP_TCP_RETRANS_MAX_SEG   30

    /** Время (мс), которе сокет находится в состоянии #LIP_TCP_SOCK_STATE_TIME_WAIT,
        чтобы повторить ответ на запрос на завершение другой стороны.
        По идее должно быть 2MSL, но это слишком большое время, если сокет
        нельзя использовать.... */
    #define LIP_TCP_TIME_WAIT         500

    /** Опредление включает использование алгоритмов slow start и congestion avoidance
        (крайне рекомендуется) */
    #define LIP_TCP_USE_SLOW_START
    /** Определение включает поддержку опции WinScale (рекомендуется) */
    #define LIP_TCP_USE_WINSCALE
    /** Определение включает поддержаку опции Timestamp (рекомендуется) */
    #define LIP_TCP_USE_TIMESTAMP
    /** Определение включает поддержку опции SACK (рекомендуется) */
    #define LIP_TCP_USE_SACK

    /** Определение включает код для режима обмена данными через буфер (очередь) сокета */
    #define LIP_TCP_USE_FIFO_MODE

    /** Определение включает код для режима передачи данных через задания на
        обмен. При определении должны быть определены также #LIP_TCP_SOCK_RX_DD_CNT
        и #LIP_TCP_SOCK_TX_DD_CNT */
    #define LIP_TCP_USE_DD_MODE
    #ifdef LIP_TCP_USE_DD_MODE
        /** Максимальное количество заданий на прием данных, которое можно поставить
            одновременно для каждого сокета */
        #define LIP_TCP_SOCK_RX_DD_CNT  4
        /** Максимальное количество заданий на передачу данных, которое можно поставить
            одновременно для каждого сокета */
        #define LIP_TCP_SOCK_TX_DD_CNT  4
    #endif

    /** Если в передаваемом сегменте данных больше, чем заданный данным определением
        размер, то данные не копируются в буфер сформированного пакета вслед за
        заголовком, а используется возможность порта (если есть) передать
        данные отдельным дескриптором без копирования */
    #define LIP_TCP_SPLIT_SEND_DATA_SIZE  32

    /** Если определено, то используется пользовтельская функция
        lip_tcp_cb_get_iss() для получения начального номера
        сегмента сокета, что позволяет, напремер, сделать номер зависимым от
        времени rtc */
    #define LIP_TCP_CB_GET_ISS

    /** Определение данного макроса включает поддержку опции #SO_KEEPALIVE.
        При этом также должны быть определены параметры #LIP_TCP_KEEPALIVE_TOUT,
        #LIP_TCP_KEEPALIVE_PROBES и #LIP_TCP_KEEPALIVE_INTERVAL
     */
    #define LIP_TCP_USE_KEEPALIVE
    #ifdef LIP_TCP_USE_KEEPALIVE
        /** Время (мс), в течение которого сокет должен находится в неактивном
            состоянии прежде чем будет выполнена первая проверка соединения,
            если для данного это разрешено с помощью опции #SO_KEEPALIVE  */
        #define LIP_TCP_KEEPALIVE_TOUT     (5*60*1000)
        /** Количество неподтвержденных проверок соединения, после которого
         *  соединение будет разорвано */
        #define LIP_TCP_KEEPALIVE_PROBES   9
        /** Интервал между проверками соединения (мс) */
        #define LIP_TCP_KEEPALIVE_INTERVAL   (75*1000)
    #endif

    /** Определение данного макроса включает поддержку активного установления
        TCP-соединений (для клиента). При его определении должны быть также
        заданы значения #LIP_TCP_TMP_PORT_FIRST и #LIP_TCP_TMP_PORT_LAST. */
    #define LIP_TCP_USE_ACTIVE_CONNECT
    #ifdef LIP_TCP_USE_ACTIVE_CONNECT
        /** TCP-порт, с которого начинается диапазон портов, выделяемых
            в качестве локального порта для ктивного установления соединений */
        #define LIP_TCP_TMP_PORT_FIRST 32768
        /** последний TCP-порт в диапазоне портов, выделяемых
            в качестве локального порта для активного установления соединений */
        #define LIP_TCP_TMP_PORT_LAST  (32768*2-1)

    #endif
#endif
/** @} */



//----------------------------------- настройки DHCP client ------------------------------------//
#define LIP_USE_DHCPC
#ifdef LIP_USE_DHCPC
    /*использование callback-а на событие "DHCP адрес получен" */
    // #define LIP_DHCPC_CONFIGURED_CB
    /*использование callback-а на событие "Ошибка получения DHCP-адреса */
    // #define LIP_DHCPC_FAILED_CB

    /* если определено, то в Discover добавляется Fast Commit опция,
     * которая позволяет серверу сразу послать Ack  */
    #define LIP_DHCPC_USE_RAPID_COMMIT

    /* если определено, то включается поддержка приема DHC_FORCERENEW
     * для реконфигурации клиента сервером  */
   #define LIP_DHCPC_USE_RECONFIGURE

    /* если определено, то в discovery и request при получении
     * адреса будет запрашиваться заданное время аренды адреса (в секундах) */
    //#define LIP_DHCPC_INITIAL_LEASE_REQUEST   600

    /* изначальный таймаут на повторную передачу */
    #define LIP_DHCPC_RETRANS_TOUT             2000

    #define LIP_DHCPC_INITIAL_DELAY_MAX  2000 //макс. задержка в мс
    /* после скольки попыток выполнить DISCOVER, считаем, что сервера нет
     * и вызываем faild callback (если определен LIP_DHCPC_FAILED_CB) */
    #define LIP_DHCPC_DISOVERY_RETRANS_FAIL 2
    /* после сокльки попыток переслать REQUEST, переходим снова
     * в состояние начальное состояние */
    #define LIP_DHCPC_REQUEST_RETRANS_CNT   4

    /* если определено, используется client-id в соответствии с rfc4361,
     * иначе - type + mac_addr */
    #define LIP_DHCPC_USE_NODE_SPEC_ID
    /* используется вместе с LIP_DHCPC_USE_NODE_SPEC_ID
     * если определено, то для получения DUID в client-id из rfc4361
     * пользователь реализует callback lip_dhcpc_get_duid_cb(),
     * иначе используется DUID типа 3 (mac+type) */
    //#define LIP_DHCPC_MANUAL_DUID
#endif


//------------------------------- настройки для IP link-local (Auto IP) -----------------//
#define LIP_USE_LINK_LOCAL_ADDR
#ifdef LIP_USE_LINK_LOCAL_ADDR
    /*********************************************************************************
     * по-умолчанию операция получения link-local адреса начинается только
     * после неудачи получения dhcp-адреса
     * если определен данный макрос, то получение link-local адреса начинается
     * сразу после link-up, даже если dhcp разрешен
     * При действительном статическом адреса поиск link-local адреса не производится
     *********************************************************************************/
    //#define LIP_LINK_LOCAL_GET_ADDR_BEFORE_DHCP_FAIL
#endif




//-------------------------------- настройки IGMP (для приема mcast) --------------------//
#define LIP_USE_IGMP
#ifdef LIP_USE_IGMP
    /*размер памяти под состояния прослушки адресов по-сокетам
     *размер памяти = (sizoof(pointer) + 8)*r + n*4, где
     *r - кол-во пар сокет-мультикаст адрес,
     *n - суммарное кол-во адресов источников в фильтрах всех пар*/
    #define LIP_IGMP_SOCKET_STATE_MEMORY_SIZE    200
    /* размер памяти под отчеты о изменении состояния прослушивания адресов */
    #define LIP_IGMP_RETRANS_STATE_MEMORY_SIZE    400
    /* размер памяти для передачи ответов на
     * group-specific и group-and-source-specific запросы */
    #define LIP_IGMP_RESPONSE_STATE_MEMORY_SIZE  200
#endif



/***************************************************************************//**
    @defgroup lip_conf_tftp Конфигурация модуля TFTP
    @brief Набор параметров, задающих конфигурацию модуля @ref lip_tftp
    @addtogroup lip_conf_tftp
    @{
********************************************************************************/

/** При определении данного макроса будет включен код модуля @ref lip_tftp для
    работы в режиме клиента */
#define LIP_USE_TFTP_CLIENT
/** При определении данного макроса будет включен код модуля @ref lip_tftp для
    работы в режиме сервера */
#define LIP_USE_TFTP_SERVER

#if defined LIP_USE_TFTP_CLIENT || defined LIP_USE_TFTP_SERVER
    /** Данный макрос включает код для передачи файлов по @ref lip_tftp
        удаленному хосту */
    #define LIP_TFTP_SUPPORT_SEND
    /** Данный макрос включает код для приема файлов по @ref lip_tftp
        от удаленного хоста */
    #define LIP_TFTP_SUPPORT_RECV

    /** Количество файлов, которое может передаваться одновременно суммарно,
        как сервером, так и клиентом */
    #define LIP_TFTP_REQUEST_CNT 6

    /** Количество повторных передач пакета, после чего посылается сообщение
        об ошибке и передача считается разорванной */
    #define LIP_TFTP_RETRANS_CNT      7
    /** Начальный таймаут на повторную передачу пакета в мс */
    #define LIP_TFTP_RETRANS_TIMEOUT_INIT  1000
    /** Максимальный таймаут на повторную передачу.
        Таймаут увеличивается с каждой последующим повтором, но не превышает
        заданного значения */
    #define LIP_TFTP_RETRANS_TIMEOUT_MAX   5000
    /** Таймаут ожидания после завершения приема файла в мс. Служит для того,
        чтобы иметь возможность ответить на дублированные пакеты в случае, если
        удаленный хост не принял последний ACK */
    #define LIP_TFTP_CPL_TIMEOUT      500
#endif

#ifdef LIP_USE_TFTP_SERVER
    /** Данный макрос включает поддержку опций TFTP: размера файла и размера
        блока. Поддержка опций реализована только для режима сервера */
    #define LIP_TFTP_SUPPORT_OPTS
#endif

/** @} */


/***************************************************************************//**
    @defgroup lip_conf_mdns Конфигурация модуля mDNS
    @brief Набор параметров, задающих конфигурацию модуля @ref lip_mdns
    @addtogroup lip_conf_mdns
    @{
********************************************************************************/

/** При определении данного макроса будет включен модуль @ref lip_mdns */
#define LIP_USE_MDNS
#ifdef LIP_USE_MDNS
    /** Максимальное количество дополнительных уникальных доменных узлов,
        добавляемых явно приложением (т.е. не считая узла, соответствующего
        имени хоста, а также узлов для регистрации сервисов через @ref lip_dns_sd) */
    #define LIP_MDNS_ADDITIONAL_UNIQUE_NODES_CNT    0

    /** Максимальное количество дополнительных разделяемых узлов, добавляемых
        явно приложением */
    #define LIP_MDNS_ADDITIONAL_SHARED_NODES_CNT    0

    /** Суммарное максимальное количество записей по всем узлам, добавляемых
        явно приложением через lip_mdns_rr_add() */
    #define LIP_MDNS_ADDITIONAL_TOTAL_RR_CNT        0

    /** Название CPU, использующееся в записи HINFO (в качестве названия ОС
        используется "lip", если не переопределено #LIP_MDNS_OS_NAME).
        Если не определено, то запись HINFO формироваться не будет */
    #define LIP_MDNS_CPU_NAME  "Cortex-M3"
#endif
/** @} */



/***************************************************************************//**
    @defgroup lip_conf_dns_sd Конфигурация модуля DNS-SD
    @brief Набор параметров, задающих конфигурацию модуля @ref lip_dns_sd
    @addtogroup lip_conf_dns_sd
    @{
********************************************************************************/
/** При определении данного макроса будет включен модуль @ref lip_dns_sd */
#define LIP_USE_DNSSD
#ifdef LIP_USE_DNSSD
    /** Максимальное общее количество экземпляров сервисов, которое можно
        зарегистрировать одновременно */
    #define LIP_DNSSD_SERVICES_CNT               1
    /** Максимальное количество пар ключ/значене в описании сервиса */
    #define LIP_DNSSD_SERVICE_MAX_TXT_KEYS       8
#endif
/** @} */

/***************************************************************************//**
    @defgroup lip_conf_mb_server Конфигурация ModbusTCP сервера
    @brief Набор параметров, задающих конфигурацию модуля @ref lip_mb_server
    @addtogroup lip_conf_mb_server
    @{
********************************************************************************/
/** При определении данного макроса будет включен модуль @ref lip_mb_server */
#define LIP_USE_MB_TCP_SERVER
#ifdef LIP_USE_MB_TCP_SERVER
    /** Количество TCP-сокетов, которое используется ModbusTCP сервером. Это
        значение определяет максимально возможное количество параллельно
        обрабатываемых клиентских подключений  */
    #define LIP_MB_TCP_SERVER_SOCKET_CNT   10

    /** Опционально. Данный макрос позволяет переопределить массив буфера
       на прием для каждого сокета ModbusTCP сервера */
    #define LIP_MB_TCP_SERVER_RECV_BUF_SIZE 1024
    /** Опционально. Данный макрос позволяет переопределить массив буфера
        на передачу для каждого сокета ModbusTCP сервера */
    #define LIP_MB_TCP_SERVER_SEND_BUF_SIZE 1024

    /** Опционально. Данный макрос позволяет ограничить максимально возможное
       количество input-регистров (при обращении к регистру с большим адресом,
       будет выдана ошибка неверного адреса без обработки). Если 0, то код
       поддержки функций для input-регистров будет исключен из стека */
    #define LIP_MB_SERVER_INPUT_REGS_MAX      65536
    /** Опционально. Данный макрос позволяет ограничить максимально возможное
        количество holding-регистров (при обращении к регистру с большим адресом,
        будет выдана ошибка неверного адреса без обработки). Если 0, то код
        поддержки функций для holding-регистров будет исключен из стека */
    #define LIP_MB_SERVER_HOLDING_REGS_MAX    65536
    /** Опционально. Данный макрос позволяет ограничить максимально возможное
        количество дискретных входов (при обращении к регистру с большим адресом,
        будет выдана ошибка неверного адреса без обработки). Если 0, то код
        поддержки функций для дискретных входов будет исключен из стека */
    #define LIP_MB_SERVER_DISCRETE_INPUTS_MAX 65536
    /** Опционально. Данный макрос позволяет ограничить максимально возможное
        количество coil-битов (при обращении к регистру с большим адресом,
        будет выдана ошибка неверного адреса без обработки). Если 0, то код
        поддержки функций для coil-битов будет исключен из стека */
    #define LIP_MB_SERVER_COILS_MAX           65536

    /** Опционально. Макрос позволяет переопределить директиву объявления буферов
        сокетов ModbusTCP сервера. Может использоваться для задания расположения
        буферов в специальной секции */
    #define LIP_MB_TCP_SERVER_BUF_MEM(var)   var
#endif
/** @} */

#endif /* LIP_CONF_H_ */

/** @} */

