/*
 * lip_eth.c
 *
 *  Created on: 31.08.2010
 *      Author: borisov
 *
 *      Файл содержит функции промежуточного уровня - между портом для mac-контроллера
 *        и протоколами сетевого уровня (IP, ARP)
 */

#include "link/lip_eth.h"
#include "lip_ll.h"
#include "lip_private.h"

#include <string.h>

#ifdef LIP_USE_RAND_GEN
    #include "misc/lip_rand.h"
#endif


#define TX_ETH_HDR ((t_lip_eth_hdr*)tx_buf)


const uint8_t g_lip_broadcast_mac[LIP_MAC_ADDR_SIZE] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

static uint8_t f_mac_addr[LIP_MAC_ADDR_SIZE];

void lip_eth_set_mac_addr(const uint8_t *mac) {
    memcpy(f_mac_addr, mac, LIP_MAC_ADDR_SIZE);
#ifdef LIP_USE_RAND_GEN
    /* генератор случайных числе инициализируется с использованием mac-адреса */
    lip_rand_init();
#endif
}

/******************************************************************************************************
 *   Передача пакета данных через ehternet. Функция получает буфер для передачи,
 *      подготавливает в нем LLC заголовок, копирует туда переданные данные и посылает пакет
 *      Функция приводит к копированию данных, в отличии от lip_eth_prepare/lip_eth_flush
 *      Используется когда данные уже сформированы в другом буфере и копировать все равно придется
 *      в частности ARP - который использует входной буфер с принятыми данными для формирования пакета)
 *   Параметры:
 *      buf  (in) - указатель на массив с данными для передачи
 *      size (in) - размер данных для передачи (без LLC заголовка)
 *      type (in) - тип/размер из LLC заголовка (определяет сетевой протокол или длину...)
 *      dest_addr - mac-адрес назначения при передаче пакета
 *   Возвращаемое значение:
 *      код ошибки
 ******************************************************************************************************/
t_lip_errs lip_eth_send(const uint8_t* buf, size_t size, t_lip_ether_type type, const uint8_t* dest_addr) {
    //получаем буфер для передачи
    uint8_t *tx_buf = lip_ll_tx_get_buf(0);
    if (tx_buf != NULL) {
        //заполняем поля
        memcpy(TX_ETH_HDR->sa, f_mac_addr, LIP_MAC_ADDR_SIZE);
        memcpy(TX_ETH_HDR->da, dest_addr, LIP_MAC_ADDR_SIZE);
        TX_ETH_HDR->type = HTON16(type);
        memcpy(&tx_buf[LIP_ETH_HDR_SIZE], buf, size); //копируем данные
        lip_ll_tx_flush(size + LIP_ETH_HDR_SIZE); //передаем кадр
    }
    return (tx_buf != NULL) ? LIP_ERR_SUCCESS : LIP_ERR_TXBUF_NOT_FOUND;
}



/***********************************************************************************************************
 *  Функция получает буфер для передачи пакета и подготавливает в нем заголовок LLC.
 *        Возвращает указатель на место в буфере сразу за заголовком
 *        - куда сетевой протокол может сохранять данные
 *      Используется совместно с lip_eth_flush для передачи пакета без лишнего копирования
 *  Параметры
 *       type      (in)  - тип/размер из LLC заголовка (определяет сетевой протокол или длину...)
 *      dest_addr (in)  - mac-адрес назначения при передаче пакета
 *      max_size  (out) - возвращает размер выделенного буфера (>0) или код ошибки (<0)
 *  Возвращаемое значение
 *      Указатель на буфер для данных протокола сетевого уровня при успехе или 0 при ошибке
 ***********************************************************************************************************/
void* lip_eth_prepare(t_lip_ether_type type, const uint8_t *dest_addr, int *max_size) {
    /* проверяем, возможна ли передача пакета в принципе */
    t_lip_errs err = lip_check_tx_enabled();
    uint8_t *tx_buf = NULL;
    if (err == LIP_ERR_SUCCESS) {
        /* получаем буфер для передачи */
        tx_buf = lip_ll_tx_get_buf(max_size);
        if (tx_buf != NULL) {
            //заполняем поля
            memcpy(TX_ETH_HDR->sa, f_mac_addr, LIP_MAC_ADDR_SIZE);
            memcpy(TX_ETH_HDR->da, dest_addr, LIP_MAC_ADDR_SIZE);
            TX_ETH_HDR->type = HTON16(type);
            if (max_size != NULL)
                *max_size -= (int)LIP_ETH_HDR_SIZE;
        } else {
            err = LIP_ERR_TXBUF_NOT_FOUND;
        }
    }

    if ((err != LIP_ERR_SUCCESS) && (max_size != NULL)) {
        *max_size = err;
    }

    return (err == LIP_ERR_SUCCESS) ? &tx_buf[LIP_ETH_HDR_SIZE] : NULL;
}


/***********************************************************************************************************
 *  Функция приводит к передаче пакета данных, который должен быть заранее подготовлен!
 *  Параметры
 *      size (in)- размер данных для передачи (без размера LLC заголовка)
 *  Возвращаемое значение:
 *     Код ошибки
 * ********************************************************************************************************/
t_lip_errs lip_eth_flush_split(size_t size, const uint8_t *data, int data_size) {
    return lip_ll_tx_flush_split(size + LIP_ETH_HDR_SIZE, data, data_size);
}


const uint8_t *lip_eth_cur_mac() {
    return f_mac_addr;
}
