/***************************************************************************//**
 @defgroup lip_port Порт
 @brief Модуль, реализующий операции, зависящие от конкретного MAC-контроллера.

  Функции, специфичные для конкретного контроллера, вынесены в модуль порта
  для данного контроллера. Все порты находятся в директории ports/"название порта"
  и включают в себя файл lip_ll_"название порта".c.

  Порт должен реализовать:
    1. Инициализацию MAC-контроллера lip_ll_init(), которая разбита на несколько стадий
    2. Выключение контроллера --- lip_ll_close()
    3. Интерфейс доступа к регистрам PHY через MII management interface
       (lip_ll_miim_read_start(), lip_ll_miim_write_start(), lip_ll_miim_rdy(),
       lip_ll_miim_data()).
    4. Передачу и прием пакетов.
    5. Опционально, при реализации multicast, поддержку входа/выхода из группы.

  Порт может также иметь свой набор настроек конфигурации. Пример этих настроек
  должен быть приведен в файле ports/_lip_port_conf.h. При этом реально используется
  только один файл конфигурации lip_conf.h, т.е. эти настройки должны быть в
  проекте либо в общем файле конфигурации, либо в отдельном, который включен в общий.

  @todo Сделать lip_ll_tx_flush_split(), чтобы можно было отслеживать
        освобождение внешнего буфера
******************************************************************************/

/***************************************************************************//**
 @addtogroup lip_port
 @{
 @file lip_ll.h
 @brief Файл содержит определения функций, которые должен реализовать
        порт для конкретного контроллера, и типов, которые используются в этих
        функциях.
 @author Borisov Alexey <borisov@lcard.ru>
*******************************************************************************/

#ifndef LIP_LL_H_
#define LIP_LL_H_



#include "lip_conf.h"
#include "lip_defs.h"


/** Флаги, управляющие начальной инициализацией стека и phy */
typedef enum {
    /** Запрет режима 100 МБит/c в режиме автоопределения параметров линка */
    LIP_INIT_FLAGS_A100_DIS       = 0x1,
    /** Инициализация phy в режиме петли для тестов */
    LIP_INIT_FLAGS_LOOPBACK       = 0x2,
    /** Запрет фильтрации входных пакетов */
    LIP_INIT_FLAGS_RX_FILTER_DIS  = 0x4
} t_lip_init_flags;

/** Этапы инициализации. При инициализации интерфейса, lip_ll_init()
    вызывается несколько раз, с соответствующим указанием этапа */
typedef enum {
    /** Стадия перед началом инициализации. На ней должны быть настроены
        пины интерфейса с Phy */
    LIP_INIT_STAGE_PHY_INTF_INIT,
    /** Начальная инициализация MAC-контрллера. Должны быть настроены
        необходимые параметры для дальнейшего использования интерфейса к
        регистрам Phy.
        Выполняется после сброса Phy (если определено #LIP_PHY_HARDWARE_RESET).*/
    LIP_INIT_STAGE_MAC_INIT,
    /** Финальная конфигурация MAC-контроллера.
        Должны быть выполнены настройки в соответствии с режимом линка
        (можно получить через lip_phy_mode() и lip_phy_speed()) и
        разрешена передача и прием */
    LIP_INIT_STAGE_MAC_START
} t_lip_init_stage;





/***************************************************************************//**
@brief Инициализация контроллера.

Данная функция вызывается несколько во время инициализации phy с разными
параметрами stage, который и определяет, что сейчас должно быть
инициализировано
@param[in] stage    Этап инициализации, определяет, что должно быть выполнено
                    по вызову функций. Одно из значений из перечисления
                    #t_lip_init_stage.
@param[in] flags   Набор флагов из #t_lip_init_flags, управляющих инициализацией
*******************************************************************************/
t_lip_errs lip_ll_init(t_lip_init_stage stage, int flags);

/***************************************************************************//**
   @brief Завершение работы с контроллером.

   Функция должна остановить все передачи, запретить работу контроллера и т.д.
 ******************************************************************************/
void lip_ll_close(void);

/***************************************************************************//**
   @brief Установка mac-адреса

   Функция передает установленный mac-адрес. Может использоваться контроллером
   для аппаратной фильтрации входных пакетов.
   @param[in] mac_addr  Установленный MAC-адрес (массив из #LIP_MAC_ADDR_SIZE байт)
 ******************************************************************************/
void lip_ll_set_mac_addr(const uint8_t *mac_addr);


/* ---------- Функции для работы с phy по MII Management интерфейсу ----------*/


/***************************************************************************//**
   @brief Запуск операции записи в регистр PHY
   @param[in] dev_addr  Адрес физического устройства
   @param[in] reg_addr  Адрес регистра
   @param[in] value     Записываемое значение
   @return Код ошибки
 ******************************************************************************/
t_lip_errs lip_ll_miim_write_start(unsigned dev_addr, unsigned reg_addr, unsigned value);
/***************************************************************************//**
   @brief Запуск операции чтения из регистра PHY
   @param[in] dev_addr  Адрес физического устройства
   @param[in] reg_addr  Адрес регистра
   @return Код ошибки
 ******************************************************************************/
t_lip_errs lip_ll_miim_read_start(unsigned dev_addr, unsigned reg_addr);
/***************************************************************************//**
   @brief Проверка, завершена ли предыдущая операция чтения/записи регистра PHY
   @return 1, если операция завершена, 0 - если интерфейс занят
 ******************************************************************************/
int lip_ll_miim_rdy(void);
/***************************************************************************//**
   @brief Получения данных, считанных из регистра PHY во время последней операции

   Функция вызывается после вызова lip_ll_miim_read_start() с последующим
   ожиданием, пока lip_ll_miim_rdy() вернет 1, чтобы прочитать считанное значение
 ******************************************************************************/
unsigned lip_ll_miim_data(void);



/* ------------------------- Функции для передачи данных ---------------------*/

/***************************************************************************//**
   @brief Получить указатель на буфер для формирования пакета.

   При наличии свободного буфера на передачу, функция должна вернуть на него
   указатель и записать в max_length максимальный его размер. Если буфера
   нет, то функция должна вернуть нулевой указатель (max_length при этом не имеет
   значения).

   @param[out] max_length  Максимальный размер данных в пакете при его наличии
   @return                 Указатель на буфер для формирования пакета на передачу
                           при его наличии. NULL в противном случае.
 ******************************************************************************/
uint8_t* lip_ll_tx_get_buf(int* max_length);

/***************************************************************************//**
   @brief Передача сформированного буфера с внешними данными

   Функция вызывается для передачи сформированного буфера, соответствующего
   последнему вызову lip_ll_tx_get_buf(). Функция не может быть вызвана без
   предварительного lip_ll_tx_get_buf(), однако lip_ll_tx_get_buf() может быть
   вызвана несколько раз до lip_ll_tx_flush().

   Также функция позволяет дополнительно передать указатель на внешний буфер,
   который должен быть включен в передаваемый пакет, после size байт из основного.

   @param size          Размер данных в подготовленном буфере  (не больше значения
                            max_length, которе вернул последний вызов lip_ll_tx_get_buf())
   @param data          Указатель на внешний буфер с данными, которые должны быть
                        переданы после данных из подготовленного
   @param data_size     Размер данных во внешнем буфере
   @return              Код ошибки
 ******************************************************************************/
t_lip_errs lip_ll_tx_flush_split(size_t size, const uint8_t* data, int data_size);


/***************************************************************************//**
   @brief Передача сформированного буфера без внешних данных

   Макрос представлет упрощенный вариант передачи данных, когда не
   нужно указывать внешний буфер
*******************************************************************************/
#define lip_ll_tx_flush(size) lip_ll_tx_flush_split(size, NULL, 0)


/* ------------------------- Функции для приема данных -----------------------*/

/***************************************************************************//**
   @brief Получение указателя на буфер с принятым кадром

   Функция должна вернуть указатель на буфер с принятым кадром, если есть
   успешно принятый полный ethernet кадр. В length должна быть возвращена
   длина кадра (только полезных данных без CRC). Весь буфер должен занимать
   непрерывную область памяти. Если не было принятого полного кадра, то
   функция должна вернуть нулевой указатель (length не имеет значения).
   Этот буфер должен сохраняться до вызова lip_ll_rx_drop().

   @param[in] length   Размер принятого кадра
   @return             Указатель на буфер с принятым кадром или NULL если
                       его нет.
 */
uint8_t* lip_ll_rx_get_buf(size_t *length);

/***************************************************************************//**
   @brief Освобождение буфера с принятым кадром

   Функция сообщает, что полученный буфер при последнем вызове lip_ll_rx_get_buf()
   больше не нужен и может быть использован портом для приема других кадров.
   Функция не может быть вызвана без предварительного вызова lip_ll_rx_get_buf().

   @return  Код ошибки
 ******************************************************************************/
t_lip_errs lip_ll_rx_drop(void);


/* ----------------- Функции для работы с групповыми передачами --------------*/
#ifdef LIP_USE_IGMP
    /***********************************************************************//**
       @brief Вход в группу с указанным MAC-адресом

       После вызова этой функции аппаратно должен быть разрешен прием
       пакетов с указанным групповым MAC-адресом

       @param[in] mac  MAC-адрес группы
       @return         Код ошибки
     */
    t_lip_errs lip_ll_join(const uint8_t* mac);
    /***********************************************************************//**
       @brief Выход из группы.

       При вызове этой функции порт может аппаратно запретить прием
       пакетов с указанным групповым MAC-адресом
       @param[in] mac  MAC-адрес группы
       @return         Код ошибки
     **************************************************************************/
    t_lip_errs lip_ll_leave(const uint8_t* mac);
#endif





#endif /* LIP_LL_H_ */

/** @}*/
