#include "ltimer.h"
#include "lip_ll.h"
#include "phy_regs.h"
#include "lip_log.h"
#include "lip_phy.h"
#include "lip_private.h"

#include "phy/lip_phy_port.h"


#ifndef LIP_PHY_MIIM_OP_TOUT
    #define LIP_PHY_MIIM_OP_TOUT  5
#endif

#ifdef LIP_PHY_LINK_TIMEOUT
    #warning parameter LIP_PHY_LINK_TIMEOUT is removed
#endif

typedef enum {
    PHY_INIT_STATE_WAIT_RST,
    PHY_INIT_STATE_RST_DONE,
    PHY_INIT_STATE_WAIT_LINK,
    PHY_INIT_STATE_WAIT_AUTONEG,    
    PHY_INIT_STATE_CONFIG,
    PHY_INIT_STATE_START,
    PHY_INIT_STATE_DONE,
    PHY_INIT_STATE_ERROR
} e_LIP_PHY_INIT_STATE;





/* Таблица с поддерживаемыми PHY */

extern t_lip_phy_info lip_phy_dp83848j;
extern t_lip_phy_info lip_phy_dp83848c;
extern t_lip_phy_info lip_phy_lan8720;

static const t_lip_phy_info* f_supported_phys[] = {
    #ifdef LIP_PHY_SUPPORT_DP83848J
        &lip_phy_dp83848j,
    #endif
    #ifdef LIP_PHY_SUPPORT_DP83848C
        &lip_phy_dp83848c,
    #endif
    #ifdef LIP_PHY_SUPPORT_LAN8720
        &lip_phy_lan8720,
    #endif
};



static e_LIP_PHY_INIT_STATE f_init_state;
static e_LIP_PHY_MODE f_mode;
static e_LIP_PHY_SPEED f_speed;

static int f_autoneg_enabled;

static const t_lip_phy_info *f_cur_phy;
/* адрес PHY устройства на шине MIIM */
static uint8_t f_phy_addr;
/* таймер - используется при инициализации интерфейса */
static t_ltimer f_init_timer;




t_lip_errs lip_phy_reg_read(uint8_t dev_addr, uint8_t reg_addr, uint16_t *value) {
    t_lip_errs err = LIP_ERR_SUCCESS;
    t_ltimer tmr;
    int rdy = 0;
    ltimer_set(&tmr, LTIMER_MS_TO_CLOCK_TICKS(LIP_PHY_MIIM_OP_TOUT));

    lip_ll_miim_read_start(dev_addr, reg_addr);
    while (!rdy && (err == LIP_ERR_SUCCESS)) {
        rdy = lip_ll_miim_rdy();
        if (!rdy && ltimer_expired(&tmr)) {
            err = LIP_ERR_INIT_PHY_MIIM_OP_TOUT;
        }
    }
    if (err == LIP_ERR_SUCCESS)
        *value = lip_ll_miim_data();
    return err;
}

t_lip_errs lip_phy_reg_write(uint8_t dev_addr, uint8_t reg_addr, uint16_t value) {
    t_lip_errs err = LIP_ERR_SUCCESS;
    t_ltimer tmr;
    int rdy = 0;
    ltimer_set(&tmr, LTIMER_MS_TO_CLOCK_TICKS(LIP_PHY_MIIM_OP_TOUT));

    lip_ll_miim_write_start(dev_addr, reg_addr, value);
    while (!rdy && (err == LIP_ERR_SUCCESS)) {
        rdy = lip_ll_miim_rdy();
        if (!rdy && ltimer_expired(&tmr)) {
            err = LIP_ERR_INIT_PHY_MIIM_OP_TOUT;
        }
    }
    return err;
}

e_LIP_PHY_MODE lip_phy_mode(void) {
    return f_mode;
}

e_LIP_PHY_SPEED lip_phy_speed(void) {
    return f_speed;
}

t_lip_errs lip_phy_init_start(int flags) {
    t_lip_errs err = LIP_ERR_SUCCESS;    
#ifdef LIP_PHY_HARDWARE_RESET
    /* аппаратный сброс phy */
    LIP_PHY_RESET_CFG();
    LIP_PHY_RESET_SET();
#endif
    err = lip_ll_init(LIP_INIT_STAGE_PHY_INTF_INIT, flags);

    if (err == LIP_ERR_SUCCESS) {
#ifdef LIP_PHY_HARDWARE_RESET
        /* запускаем таймер на сброс */
        ltimer_set(&f_init_timer, LTIMER_MS_TO_CLOCK_TICKS(LIP_PHY_RESET_TIME));
        f_init_state = PHY_INIT_STATE_WAIT_RST;
#else
        f_init_state = PHY_INIT_STATE_RST_DONE;
#endif
    }
    return err;
}

t_lip_errs lip_phy_init_pull(int flags) {
    t_lip_errs err = LIP_ERR_SUCCESS;
    uint16_t phy_reg;

    switch (f_init_state) {
#ifdef LIP_PHY_HARDWARE_RESET
        case PHY_INIT_STATE_WAIT_RST:
            if (ltimer_expired(&f_init_timer)) {
                LIP_PHY_RESET_CLR();
                f_init_state = PHY_INIT_STATE_RST_DONE;
            } else {
                break;
            }
#endif
        case PHY_INIT_STATE_RST_DONE:
            err = lip_ll_init(LIP_INIT_STAGE_MAC_INIT, flags);
            if (err == LIP_ERR_SUCCESS) {
                f_cur_phy = NULL;
                /* поиск phy устройства с нужными ID по всем адресам */
                for (uint8_t phy_addr = 0; (phy_addr < 32) && (f_cur_phy==NULL); phy_addr++) {
                    uint16_t id1, id2;
                    size_t i;

                    err = lip_phy_reg_read( phy_addr, LIP_PHY_REG_PHYIDR1, &id1);
                    if (err == LIP_ERR_SUCCESS)
                        err = lip_phy_reg_read( phy_addr, LIP_PHY_REG_PHYIDR2, &id2);
                    if (err == LIP_ERR_SUCCESS) {
                        for (i=0; i < sizeof(f_supported_phys)/sizeof(f_supported_phys[0])
                             && (f_cur_phy==NULL); i++) {
                            if (((id1&0xFFFF) == f_supported_phys[i]->id1) &&
                                    ((id2 & f_supported_phys[i]->id2_msk) ==
                                     f_supported_phys[i]->id2)) {
                                f_cur_phy = f_supported_phys[i];
                                f_phy_addr = phy_addr;
                                lip_printf(LIP_LOG_MODULE_PHY, LIP_LOG_LVL_DEBUG_LOW, 0,
                                        "lip phy: found phy %s with addr %d\n", f_cur_phy->name, f_phy_addr);
                            }
                        }
                    }
                }

                if (f_cur_phy == NULL) {
                    err = LIP_ERR_INIT_PHY_NOTFOUND;
                    lip_printf(LIP_LOG_MODULE_PHY, LIP_LOG_LVL_ERROR_FATAL, res,
                            "lip phy: cannot find ethernet phy!\n");
                }
            }

            if (err == LIP_ERR_SUCCESS) {
                if (f_cur_phy->init!=NULL)
                    err = f_cur_phy->init(f_phy_addr, flags);

                if (err == LIP_ERR_SUCCESS) {
                    if (!(flags & LIP_INIT_FLAGS_LOOPBACK)) {
                        /* Разрешаем только 10 Mbit/s при соответствующем флаге */
                        if (flags & LIP_INIT_FLAGS_A100_DIS) {
                            err = lip_phy_reg_write( f_phy_addr, LIP_PHY_REG_ANAR,
                                                     1 | ANAR_10BT | ANAR_10BT_FULL);
                        }
                        if (err==LIP_ERR_SUCCESS) {
                            /* запускаем таймер на ожидание завершения AutoNegotiation */
                            err = lip_phy_reg_write( f_phy_addr, LIP_PHY_REG_BMCR, BMCR_AN | BMCR_RE_AN);                            
                            f_autoneg_enabled = 1;
                        }
                    } else {
                        /* в режиме кольца AutoNegotiation не нужен - явно
                         * устанавливаем режим и ждем link up */
                        f_autoneg_enabled = 0;
                        err = lip_phy_reg_write(f_phy_addr, LIP_PHY_REG_BMCR,
                                                (flags & LIP_INIT_FLAGS_A100_DIS ?
                                                0 : BMCR_SPEED_100)
                                                | BMCR_DUPLEX | BMCR_LOOPBACK);
                    }
                }

                if (err == LIP_ERR_SUCCESS)
                    f_init_state = PHY_INIT_STATE_WAIT_LINK;
            }
            break;
        case PHY_INIT_STATE_WAIT_LINK:
            /* проверка link_up */
            err = lip_phy_reg_read( f_phy_addr, LIP_PHY_REG_BMSR, &phy_reg );
            if (err == LIP_ERR_SUCCESS) {
                if (phy_reg & BMSR_LINK_ESTABLISHED) {
                    lip_printf(LIP_LOG_MODULE_PHY, LIP_LOG_LVL_DEBUG_MEDIUM, 0,
                            "lip phy: link up\n");
                    if (f_autoneg_enabled) {
                        f_init_state =PHY_INIT_STATE_WAIT_AUTONEG;
                        ltimer_set(&f_init_timer, LTIMER_MS_TO_CLOCK_TICKS(LIP_PHY_AUTONEG_TIMEOUT));
                    } else {
                        f_init_state = PHY_INIT_STATE_CONFIG;
                    }
                }
            }
            break;
        case PHY_INIT_STATE_WAIT_AUTONEG:
            /* ожидание завершения AutoNegotiation */
            err = lip_phy_reg_read( f_phy_addr, LIP_PHY_REG_BMSR, &phy_reg );
            if (err == LIP_ERR_SUCCESS) {
                if (phy_reg & BMSR_AUTO_DONE) {
                    f_init_state = PHY_INIT_STATE_CONFIG;
                    lip_printf(LIP_LOG_MODULE_PHY, LIP_LOG_LVL_DEBUG_LOW, 0,
                               "lip phy: link autonegotiation done\n");
                } else {
                    if (ltimer_expired(&f_init_timer)) {
                        err = LIP_ERR_INIT_PHY_AUTONEG_TIMEOUT;
                        lip_printf(LIP_LOG_MODULE_PHY, LIP_LOG_LVL_WARNING_MEDIUM, err,
                                "lip phy: link autonegotiation error!\n");
                    }
                }
            }
            break;

        case PHY_INIT_STATE_CONFIG:
            /* получаем реально установленный режим, чтобы правильно
             * настроить MAC-контроллер и устанавливаем его явно,
             * чтобы не мог измениться */
            err = f_cur_phy->get_mode(f_phy_addr, &f_speed, &f_mode);
            if (err == LIP_ERR_SUCCESS) {
                phy_reg = 0;

                if (f_speed == LIP_PHY_SPEED_100) {
                    phy_reg |= BMCR_SPEED_100;
                    lip_printf(LIP_LOG_MODULE_PHY, LIP_LOG_LVL_PROGRESS, 0,
                            "lip phy: link speed: 100 Mbit/s\n");
                } else {
                    lip_printf(LIP_LOG_MODULE_PHY, LIP_LOG_LVL_PROGRESS, 0,
                            "lip phy: link speed: 10 Mbit/s\n");
                }

                if (f_mode == LIP_PHY_MODE_FULL_DUPLEX)  {
                    phy_reg |= BMCR_DUPLEX;
                    lip_printf(LIP_LOG_MODULE_PHY, LIP_LOG_LVL_PROGRESS, 0,
                            "lip phy: link mode: full duplex\n");
                } else {
                    lip_printf(LIP_LOG_MODULE_PHY, LIP_LOG_LVL_PROGRESS, 0,
                            "lip phy: link mode: half duplex\n");
                }

                if (flags & LIP_INIT_FLAGS_LOOPBACK)
                    phy_reg |= BMCR_LOOPBACK;

                err = lip_phy_reg_write( f_phy_addr, LIP_PHY_REG_BMCR, phy_reg);
                f_init_state = PHY_INIT_STATE_START;
            }
            break;
        case PHY_INIT_STATE_START:
            lip_ll_set_mac_addr(lip_eth_cur_mac());
            err = lip_ll_init(LIP_INIT_STAGE_MAC_START, flags);
            if (err == LIP_ERR_SUCCESS) {
                f_init_state = PHY_INIT_STATE_DONE;
            }
            break;
        default:
            err = LIP_ERR_INIT_STATE;
            break;
    }

    if (err!=LIP_ERR_SUCCESS) {
        f_init_state = PHY_INIT_STATE_ERROR;
    }

    return err != LIP_ERR_SUCCESS ? err :
           f_init_state == PHY_INIT_STATE_DONE ? LIP_ERR_SUCCESS  :
                                                 LIP_ERR_INIT_IN_PROGRESS;
}

int lip_phy_get_link_status(void) {
    uint16_t phy_reg = 0;
    lip_phy_reg_read( f_phy_addr, LIP_PHY_REG_BMSR, &phy_reg );
    return (phy_reg & BMSR_LINK_ESTABLISHED);
}


