/* LPC4333 memory map:

   SRAM
   0x10000000 32K   Local SRAM
   0x10080000 40K   Local SRAM, top 8K retained in power-down mode
   0x20000000 32K   AHB SRAM
   0x20008000 16K   AHB SRAM
   0x2000C000 16K   AHB/ETB SRAM
   FLASH
   0x1A000000 256K  Flash bank A
   0x1B000000 256K  Flash bank B
   SPIFI
   0x14000000 64M
   STATIC EXTERNAL MEMORY (EMC)
   0x1C000000 4x16M (CS0, CS1, CS2, CS3)

   Bit banding:
   0x20000000 (1M) @ 0x22000000
   0x40000000 (1M) @ 0x42000000
*/

MEMORY
    {
    FLASH_A (rx)    : ORIGIN = 0x1A000000+FLASH_START_OFFSET, LENGTH = 256k
    FLASH_B (rx)    : ORIGIN = 0x1B000000, LENGTH = 256k

    SRAM_1 (rwx)    : ORIGIN = 0x10000000, LENGTH = 32k
    SRAM_2 (rwx)    : ORIGIN = 0x10080000, LENGTH = 40k
    SRAM_AHB        : ORIGIN = 0x20000000, LENGTH = 32k /* 32 + 16 + 16 */
    SRAM_AHB2       : ORIGIN = 0x20008000, LENGTH = 16k
    SRAM_AHB3       : ORIGIN = 0x2000C000, LENGTH = 16k
    SDRAM           : ORIGIN = 0x28000000, LENGTH = 32M
    }

/* Region aliases can be used to quickly remap to a different configuration.
   E.g. keeping code and data on different buses may be faster. */

REGION_ALIAS("CODE_ROM",    FLASH_A);   /* Program code */
REGION_ALIAS("DATA_ROM",    FLASH_A);   /* Const data and initializers */
REGION_ALIAS("DATA_RAM",    SRAM_2);
REGION_ALIAS("STACK_RAM",   SRAM_2);  /* System stack (at the beginning of the region) */
REGION_ALIAS("STACK_RAM_M0",   SRAM_AHB2);
REGION_ALIAS("CODE_RAM_M0",   SRAM_AHB2);
REGION_ALIAS("DATA_RAM_M0",   SRAM_AHB2);
REGION_ALIAS("M0_RAM",   SRAM_2);
REGION_ALIAS("SRAM_ETH",    SRAM_AHB3);
REGION_ALIAS("SRAM_FPGA_BUF", SRAM_AHB2);
REGION_ALIAS("IFACE_BUF",   SRAM_1);
REGION_ALIAS("IFACE_OUT_BUF", SRAM_AHB);


/* Section Definitions */
SECTIONS {
    /* -------- ROM -------- */

    /* To reserve space for bootloader, pass --section-start=.text=... to linker (gcc -Wl,...) */
    .text : ALIGN(4) {
        FILL(0xFF)
        _text = .;
        KEEP(*(.isr_vector .isr_vector.*))
        . = _text + 0x200;
        KEEP(*(.appl_info))
        *(.text .text.*)
        _etext = .;
    } > CODE_ROM

    .rodata : ALIGN(4) {
        /* Static data init table */
        . = ALIGN(4);
        __data_init_tab_start = .;
        LONG(LOADADDR(.data));
        LONG(ADDR    (.data));
        LONG(SIZEOF  (.data));
        LONG(LOADADDR(.text_m0));
        LONG(ADDR    (.text_m0));
        LONG(SIZEOF  (.text_m0));
        LONG(LOADADDR(.data_m0));
        LONG(ADDR    (.data_m0));
        LONG(SIZEOF  (.data_m0));
        __data_init_tab_end = .;

        __bss_init_tab_start = .;
        LONG(ADDR    (.bss));
        LONG(SIZEOF  (.bss));
        LONG(ADDR    (.bss_m0));
        LONG(SIZEOF  (.bss_m0));
        __bss_init_tab_end = .;
        
        *(.gnu.linkonce.t.*)
        *(.glue_7)
        *(.glue_7t)
        *(.gcc_except_table)
        *(.rodata .rodata*)
        *(.gnu.linkonce.r.*) 

        . = ALIGN(4);
        *(.ARM.extab* .gnu.linkonce.armextab.*) 
    } > DATA_ROM


  .ARM.exidx : {
    __exidx_start = .;
    *(.ARM.exidx* .gnu.linkonce.armexidx.*)
    __exidx_end = .;
  } > DATA_ROM
  
    /* -------- END-OF-ROM marker (for checksum of text + rodata + datainits ) -------- */
    .rom_end : { /* Checksum of _text to _rom_end == checksum of .bin file (if CODE_ROM = DATA_ROM) */
        _rom_end = .;
    } > CODE_ROM

    /* -------- STACK -------- */

    /* Stack at the beginning of SRAM 2 */
    .stackarea (NOLOAD) : {
        . = ALIGN(8);
         *(.stackarea .stackarea.*)
        . = ALIGN(8);
    } > STACK_RAM
   
    /* -------- MAIN RAM -------- */

    .data : ALIGN(4) {
         _data = .;
         *(vtable vtable.*)
         *(.data .data.*)
         *(.gnu.linkonce.d*)
         *(.ram_func)
        . = ALIGN(4);
        _edata = . ;
    } > DATA_RAM AT > DATA_ROM
 
    .bss (NOLOAD) : ALIGN(4) {
        _bss = . ;
        *(.bss .bss.*)
        *(.gnu.linkonce.b*)
        *(COMMON)
        . = ALIGN(4);
        _ebss = . ;
    } > DATA_RAM




    /* -------- M0 CODE (in RAM) -------- */
    .text_m0 : ALIGN(4096) /* M0APPMAP register cannot have nonzero bits [11:0] */ {
        FILL(0xFF)
        _text_m0 = .;
        KEEP(*(.isr_vector_m0 .isr_vector_m0.*))
        *(.text_m0 .text_m0.*)
        _etext_m0 = .;
    } > CODE_RAM_M0 AT > CODE_ROM

    /* -------- STACK -------- */

    /* Stack at the beginning of SRAM 2 */
    .stackarea_m0 (NOLOAD) : {
        . = ALIGN(8);
         *(.stackarea_m0 .stackarea_m0.*)
        . = ALIGN(8);
    } > STACK_RAM_M0

    /* -------- M0 DATA -------- */
    .data_m0 : ALIGN(4){
         _data_m0 = .;
        *(.data_m0 .data_m0.*)
        . = ALIGN(4);
        _edata_m0 = . ;
    } > DATA_RAM_M0 AT > DATA_ROM

    .bss_m0 (NOLOAD) : ALIGN(4) {
        _bss_m0 = . ;
        *(.bss_m0 .bss_m0.*)
        . = ALIGN(4);
        _ebss_m0 = . ;
    } > DATA_RAM_M0







    .eth_ram (NOLOAD): {
        *(.eth_ram)
    } > SRAM_ETH

    .fpga_buf (NOLOAD): {
        *(.fpga_buf)
    } > SRAM_FPGA_BUF

    .iface_buf (NOLOAD): {
        *(.iface_buf)
    } > IFACE_BUF

    .iface_out_buf (NOLOAD): {
        *(.iface_out_buf)
    } > IFACE_OUT_BUF

    .sdram (NOLOAD) : {
	_sdram = .;
        *(.sdram)
	_esdram = .;
    } > SDRAM
  
    



    /* PROVIDE (end = .); */


/* -------- DEBUG SECTIONS -------- */
 
  /* Stabs debugging sections.  */
  .stab          0 : { *(.stab) }
  .stabstr       0 : { *(.stabstr) }
  .stab.excl     0 : { *(.stab.excl) }
  .stab.exclstr  0 : { *(.stab.exclstr) }
  .stab.index    0 : { *(.stab.index) }
  .stab.indexstr 0 : { *(.stab.indexstr) }
  .comment       0 : { *(.comment) }
  /* DWARF debug sections.
     Symbols in the DWARF debugging sections are relative to the beginning
     of the section so we begin them at 0.  */
  /* DWARF 1 */
  .debug          0 : { *(.debug) }
  .line           0 : { *(.line) }
  /* GNU DWARF 1 extensions */
  .debug_srcinfo  0 : { *(.debug_srcinfo) }
  .debug_sfnames  0 : { *(.debug_sfnames) }
  /* DWARF 1.1 and DWARF 2 */
  .debug_aranges  0 : { *(.debug_aranges) }
  .debug_pubnames 0 : { *(.debug_pubnames) }
  /* DWARF 2 */
  .debug_info     0 : { *(.debug_info .gnu.linkonce.wi.*) }
  .debug_abbrev   0 : { *(.debug_abbrev) }
  .debug_line     0 : { *(.debug_line) }
  .debug_frame    0 : { *(.debug_frame) }
  .debug_str      0 : { *(.debug_str) }
  .debug_loc      0 : { *(.debug_loc) }
  .debug_macinfo  0 : { *(.debug_macinfo) }
  /* SGI/MIPS DWARF 2 extensions */
  .debug_weaknames 0 : { *(.debug_weaknames) }
  .debug_funcnames 0 : { *(.debug_funcnames) }
  .debug_typenames 0 : { *(.debug_typenames) }
  .debug_varnames  0 : { *(.debug_varnames) }

}

/*** EOF ***/
